/* Othello
 * Le joueur 1 joue avec les pions noirs
 * Le joueur 2 joue avec les pions blancs
 */

#include "othello.h"

/* Rempli le plateau pour dmarrer la partie */
void remplissage_plateau(char plateau[8][8])
{
    int i,j;
    for ( i=0 ; i < 8 ; i++ )
	for ( j=0 ; j < 8 ; j++ )
	    if ( i == 3 ) {
		switch(j)
		{
		    case 3: plateau[i][j] = BLANC; break;
		    case 4: plateau[i][j] = NOIR; break;
		    default: plateau[i][j] = VIDE; break;
		}
	    } else if ( i == 4 ) {
		switch(j)
		{
		    case 3: plateau[i][j] = NOIR; break;
		    case 4: plateau[i][j] = BLANC; break;
		    default: plateau[i][j] = VIDE; break;
		}
	    } else
		plateau[i][j] = 0;
    return;	
}

/* Retourne le nombre de pices retournes */
int lignes (char plateau[8][8], int x, int y, int xdir, int ydir, int joueur, int retourne)
{
    int i,r=0;

    for ( i=1 ; x+i*xdir >= 0 && x+i*xdir <= 7 && y+i*ydir >= 0 && y+i*ydir <= 7 ; i++ )
    {
	if ( plateau[y+i*ydir][x+i*xdir] == joueur ) break;
	if ( plateau[y+i*ydir][x+i*xdir] == VIDE 
	    || ( x+i*xdir == 7 && xdir != 0 )
	    || ( y+i*ydir == 7 && ydir != 0 )
	    || ( x+i*xdir == 0 && xdir != 0 )
	    || ( y+i*ydir == 0 && ydir != 0 ) ) { r=0; break; }
	r++;
    }

    if ( retourne == 1 && r != 0 )
    {
	int z=r;
	for ( i=1 ; z > 0 ; i++,z-- )
	    if ( retourne == 1 )
		plateau[y+i*ydir][x+i*xdir] = joueur;
    }

    return r;
}

/* Vrification ( et retournement des pices ? ) dans les huits directions */
int othello (char plateau[8][8], int x, int y, int joueur, int retourne)
{
    int r=0,i,j;

    for ( i=-1 ; i <= 1 ; i++ )
	for ( j=-1 ; j <= 1 ; j++ )
	    if ( !(j == 0 && i == 0) ) // Direction nulle
		r += lignes(plateau,x,y,j,i,joueur,retourne);
    
    return r;
}

/* Jeu */
int jeu (MlvType *ecran, char plateau[8][8], int joueur)
{
    int x,y,r;

    // On demande une case valide
    do {
	mouse_wait(&x,&y,ecran);
    } while ( y > TCASES*8
	     || plateau[y/TCASES][x/TCASES] != VIDE
	     || othello(plateau,x/TCASES,y/TCASES,joueur,0) == 0 );


    // On retourne les pices
    r = othello(plateau,x/TCASES,y/TCASES,joueur,1);
    plateau[y/TCASES][x/TCASES] = joueur;

    return r;
}

/* Initialisation et dmarrage de la partie */
int main (void)
{
    char plateau[8][8];
    int score_j1=2,score_j2=2,joueur=1,score;
    MlvType ecran;

    /* Initialisation */
    create_window(DISPLAY,TCASES*8,TCASES*8+20,&ecran);
    fenetre_plateau(&ecran);
    remplissage_plateau(plateau);

    /* Boucle de jeu */
    do {
	// Affichage
	affichage(plateau,&ecran);
	affichage_scores(&ecran,score_j1,score_j2);
	affichage_joueur(joueur,&ecran);
	// Jeu
	score = jeu(&ecran,plateau,joueur);
	// Changement de joueur et mise  jour des scores
	if ( joueur == 1 )
	{
	    joueur = 2;
	    score_j1 += score+1;
	    score_j2 -= score;
	} else {
	    joueur = 1;
	    score_j2 += score+1;
	    score_j1 -= score;
	}
    } while ( score_j1+score_j2 < 64 || score_j1 == 0 || score_j2 == 0 );
    
    /* Fin de partie */
    affichage(plateau,&ecran);
    affichage_scores(&ecran,score_j1,score_j2);
    affichage_joueur(joueur,&ecran);
    gameover(&ecran);

    return 0;
}
